import sys,re,json,os,shutil
import argparse

def error(sentences):
    print("-"*30)
    print("Error:")
    print("\n".join(sentences))
    print("autoCBEI.py exit...")
    print("-"*30)
    sys.exit()

def mainAutoCBEI():
    #Set the Base editor parameter
    # [PAM, spacer length, edit beg, edit end, direction]
    # Direction refers to spacer at the 5' or 3' end of PAM sequence (5 or 3, respectively). 
    beinfos = {
        "BE":["NGG",20,4,8,5],
        "YE1-BE3":["NGG",20,5,7,5],
        "EE-BE3":["NGG",20,5,6,5],
        "YEE-BE3":["NGG",20,6,6,5],
        "VQR-BE3":["NGAN",20,4,11,5],
        "VRER-BE3":["NGCG",20,3,10,5],
        "SaBE":["NNGRRT",21,3,12,5],
        "Sa(KKH)-BE3":["NNNRRT",21,3,12,5],
        "Cas12a–BE":["TTTV",20,10,12,3],
        "Target-AID":["NGG",20,2,4,5],
        "Target-AID-NG":["NG",20,2,4,5],
        "xBE3":["NG",20,4,8,5],
        "BE-PLUS":["NGG",20,4,14,5]
    }



    try:
        from Bio import SeqIO
    except ImportError:
        error(["The \"biopython\" package was not found.",
            "Please install \"biopython\"."])

    try:
        import matplotlib
    except ImportError:
        error(["The \"matplotlib\" package was not found.",
            "Please install \"matplotlib\"."])

    try:
        from autocbei.cbei import cbei,stat
    except ImportError:
        error(["The \"cbei\" package was not found.",
            "Please reinstall \"autocbei\" with \"pip install --upgrade autocbei\"."])
    
    from autocbei.cbei.judges import judgeBEF
    # demoPath=os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])),"Bacillus_subtilis.part500.cds.all.fa")

    parser = argparse.ArgumentParser(description='Enter fasta file of CDSs,\
            output base editor\'s potential editing site and statistics information.\n \
            The demo CDS Fasta file: "[PATHON_HOME]/site-packages/autocbei/Bacillus_subtilis.part500.cds.all.fa".')
    parser.add_argument("cds",metavar='CDS.fasta',help="CDSs in fasta format.",type=str)
    parser.add_argument("-bef",help="[Optional] The file containing the base editor definition, seperate with \",\". eg: \n \
                        \"BE,NGG,20,4,8,5\", that is, \"name, PAM, spacer length, edit beg, edit end, PAM at 5' or 3' of spacer\". \
                        13 base editors are included by default. \
                        ",type=str)
    # parser.add_argument("-be",help="[Optional] Define a base editor, seperate with \",\". eg: \n \
                        # \"BE,NGG,20,4,8,5\", that is, \"name, PAM, spacer length, edit beg, edit end, PAM at 5' or 3' of spacer\"\
                        # ",type=str)
                        
    parser.add_argument("-ns","--nostat",help="Only run CBEI design without statistics and plot.",action="store_true")
    parser.add_argument("-o","--outprefix",help="Directory prefixes can be customized. Default: \"CBEI\" (CBEIRaw, CBEIPlot, CBEIRes). ", type=str)
    args = parser.parse_args()
    # 1. The potential editing sites for the base editor are calculated, 
    #     and each base editor generates a separate cbei file.

    pre="CBEI"
    if (args.outprefix):
        pre=args.outprefix
    
    if (args.bef):
        beinfos=judgeBEF(args.bef)
        print("Successfully imported "+str(len(beinfos))+" base editors from file: "+args.bef)


    rawPath=pre+"Raw"
    plotPath=pre+"Plot"
    resPath=pre+"Stat"

    print("Input file: "+args.cds)
    print("Output directory: "+rawPath)
    print("Base editors: ")
    print("\tBE\tPAM\tSpacer\tEditBegin\tEditEnd\tDirection")
    for key in beinfos.keys():
        print("\t"+key,end="\t")
        for t in beinfos[key]:
            print(t,end="\t")
        print()

    seqDict = SeqIO.index(args.cds, "fasta")
    tName=os.path.basename(args.cds).split(".")
    fileName=tName[0]
    for key in beinfos:
        print("Start calculating: "+key)
        cbei.runBatch(key, beinfos[key], seqDict,rawPath)
    print("Calculate complete!")

    if (not args.nostat):
        print("Begin statistics...")
        print("The statistics directory: "+resPath)
        print("The plot directory: "+plotPath)

        stat.statCBEI(seqDict,rawPath,plotPath,resPath,fileName)
        print("CBEI statistics complete")

if __name__ == "__main__":
    mainAutoCBEI()



def mainCBEI():
    from sys import version_info
    if version_info.major != 3:
        print("Warning!")
        print('\tYour python version is '+str(version_info.major)+'.'+str(version_info.minor))
        print('\t"autocbei" relies on python3, please use python3 (Recommended python>=3.6.0)!')
        print('autocbei exit...')
        return
    from autocbei.cbei.autoCBEI import mainAutoCBEI
    mainAutoCBEI()


if __name__ == "__main__":
    mainCBEI()


import sys,re,json,os,shutil
from Bio import SeqIO
from Bio.SeqUtils import GC

baseT = {
    "A": "T",
    "T": "A",
    "C": "G",
    "G": "C",
    "U": "A",
    "a": "t",
    "t": "a",
    "c": "g",
    "g": "c",
    "u": "a",
    "N": "N",
    "n": "n",
    "R": "Y",
    "Y": "R",
    "M": "K",
    "K": "M",
    "W": "W",
    "S": "S",
    "H": "D",
    "D": "H",
    "B": "V",
    "V": "B",
    "r": "y",
    "y": "r",
    "m": "k",
    "k": "m",
    "w": "w",
    "s": "s",
    "h": "d",
    "d": "h",
    "b": "v",
    "v": "b"
}


def getPAMReg(pam):
    spArr = list(pam)
    for s in range(len(spArr)):
        if (re.match(r"N", spArr[s], re.I)):
            spArr[s] = r"\w"
        elif (re.match(r"R", spArr[s], re.I)):
            spArr[s] = "[AGR]"
        elif (re.match(r"Y", spArr[s], re.I)):
            spArr[s] = "[CTY]"
        elif (re.match(r"M", spArr[s], re.I)):
            spArr[s] = "[ACM]"
        elif (re.match(r"K", spArr[s], re.I)):
            spArr[s] = "[GTK]"
        elif (re.match(r"S", spArr[s], re.I)):
            spArr[s] = "[GCS]"
        elif (re.match(r"W", spArr[s], re.I)):
            spArr[s] = "[ATW]"
        elif (re.match(r"H", spArr[s], re.I)):
            spArr[s] = "[ATCWMYH]"
        elif (re.match(r"B", spArr[s], re.I)):
            spArr[s] = "[GTCKSYB]"
        elif (re.match(r"V", spArr[s], re.I)):
            spArr[s] = "[GACRSMV]"
        elif (re.match(r"D", spArr[s], re.I)):
            spArr[s] = "[GATRKWD]"
        treg = '(?=('+"".join(spArr)+'))'
    return re.compile(treg, re.I)

def tranStrand(seq):
    rSeq = seq[::-1]
    newSeq = ""
    for i in range(len(rSeq)):
        newSeq += baseT[rSeq[i]]
    return newSeq


def changeCodon(codon, pos, change):
    if (pos == 1):
        return change+codon[pos:]
    elif (pos == len(codon)):
        return codon[0:-1]+change
    else:
        return codon[0:pos-1]+change+codon[pos:]


def addLabel(seq, addPos, addStr, reg):
    count = -1
    newSeq = ''
    errLab = 1
    for i in range(len(seq)):
        if (re.match(reg, seq[i])):
            newSeq = newSeq+seq[i]
            continue
        else:
            count += 1
        if (count == addPos):
            newSeq = newSeq+addStr+seq[i]
            errLab = 0
        else:
            newSeq = newSeq+seq[i]
    if (errLab):
        newSeq = newSeq+addStr
    return newSeq


def calSapcer(seq, regPos, beinfo, endReg, mode):
    preStop = []
    pam=beinfo[0]
    spLen=beinfo[1]
    editWin=beinfo[2:4]
    direct=beinfo[4]
    for tpos in regPos:
        pamInfo = {}
        pamInfo['mode'] = mode
        pamPos = tpos.start()+1
        b = -1
        e = -1
        spb = -1
        spe = -1
        if (direct == 5):
            if (pamPos-spLen > -1):
                spb = pamPos-spLen
                spe = pamPos-1
            else:
                continue
            b = spb+editWin[0]-1
            e = spb+editWin[1]-1
        else:
            if (pamPos+spLen > len(seq)):
                continue
            spb = pamPos+len(pam)
            spe = pamPos+len(pam)+spLen-1
            b = pamPos+editWin[0]+len(pam)-1
            e = pamPos+editWin[1]+len(pam)-1
        pamInfo['pamSeq'] = seq[tpos.start():(tpos.start()+len(pam))]
        if (mode == 'Minus'):
            pamInfo['spacerRange'] = str(
                len(seq)-spb+1)+"-"+str(len(seq)-spe+1)
            pamInfo['editRange'] = str(len(seq)-b+1)+"-"+str(len(seq)-e+1)
            pamInfo['pamRange'] = str(
                len(seq)-pamPos+2-len(pam))+"-"+str(len(seq)-pamPos+1)
        else:
            pamInfo['spacerRange'] = str(spb)+"-"+str(spe)
            pamInfo['editRange'] = str(b)+"-"+str(e)
            pamInfo['pamRange'] = str(pamPos)+"-"+str(pamPos+len(pam)-1)

        pamInfo['pureSpacer'] = seq[spb-1:spe]
        # print(pureSpacer)
        longspb = spb
        longspe = spe

        if (spb % 3 == 0):
            longspb = longspb - 2
        elif (spb % 3 == 2):
            longspb = longspb - 1

        if (spe % 3 == 1):
            longspe = longspe + 2
        elif (spe % 3 == 2):
            longspe = longspe + 1

        longSpacer = seq[longspb-1:longspe]
        tedit = []
        tcodonall = []
        for tsp in range(int(len(longSpacer)/3)):
            tcodon = longSpacer[tsp*3:tsp*3+3]
            if (re.match(endReg, tcodon)):
                tedit.append(longspb+tsp*3)
            tcodonall.append(tcodon)
        tcodonORI = tcodonall.copy()

        editPos = ""
        location = -1.1
        editPatten = ""
        tios = []
        for ttedit in tedit:
            teditCodonIndex = int((ttedit-longspb)/3)
            newCodon=""
            for subI in range(3):
                ttIndex = subI+ttedit
                if (ttIndex >= b and ttIndex <= e):
                    if (re.match(r'C', tcodonORI[teditCodonIndex][subI], re.I)):
                        # if (mode == 'Minus'):
                        #     print(tcodonORI[teditCodonIndex]+"\t"+tcodonORI[teditCodonIndex][subI]+"\t"+str(subI))
                        tios.append(ttIndex)
                        newCodon=newCodon+"(c->t)"
                        # tcodonORI[teditCodonIndex][subI]="(c->t)"
                        # tcodonall[teditCodonIndex] = changeCodon(
                        #     tcodonORI[teditCodonIndex], subI+1, "(c->t)")
                    else:
                        newCodon=newCodon+tcodonORI[teditCodonIndex][subI]
                else:
                    newCodon=newCodon+tcodonORI[teditCodonIndex][subI]
            tcodonall[teditCodonIndex]=newCodon

        if (not tios):
            continue

        teditpatten = []
        for ii in range(len(tios)):
            teditpatten.append(seq[tios[ii]-2:tios[ii]])
        editPatten = ",".join(teditpatten)

        if (mode == 'Minus'):
            location = (len(seq)-tios[0]+1) / len(seq)
        else:
            location = tios[0]/len(seq)

        for ios in range(len(tios)):
            if (mode == 'Minus'):
                tios[ios] = str(len(seq)-tios[ios]+1)
            else:
                tios[ios] = str(tios[ios])

        # editPos=str(len(seq)-tios[1]+1)+","+str(len(seq)-tios[0]+1)
        editPos = ",".join(tios)

        if (not editPos):
            continue

        detailSeq = ",".join(tcodonall)

        skipReg = re.compile(r',|\{|\}|\[|\]|\)|\-|\>|[c,t]')

        detailSeq = addLabel(detailSeq, abs(
            longspb-spb)+editWin[0]-1, '[', skipReg)
        detailSeq = addLabel(detailSeq, abs(
            longspb-spb)+editWin[1], ']', skipReg)

        if (direct == 5):
            detailSeq = addLabel(detailSeq, abs(longspb-spb), '{', skipReg)
            detailSeq = addLabel(detailSeq, len(
                longSpacer)-abs(longspe-spe), '}|', skipReg)
            detailSeq = detailSeq+',' + \
                seq[longspe:longspe+len(pam)-(abs(longspe-spe))]
        else:
            detailSeq = addLabel(detailSeq, abs(longspb-spb), '|{', skipReg)
            detailSeq = addLabel(detailSeq, len(
                longSpacer)-abs(longspe-spe)+2, '}', skipReg)
            detailSeq = seq[longspb -
                            len(pam)+abs(longspb-spb)-1:longspb-1]+","+detailSeq
        # print(detailSeq.upper())
        pamInfo['detailSpacer'] = detailSeq.upper()

        pamInfo['editpos'] = editPos
        pamInfo['location'] = location
        pamInfo['editpatten'] = editPatten
        preStop.append(pamInfo)
    return preStop

def runBatch(bename, beinfo, seqDict,rawPath):
    if ( not os.path.exists(rawPath)):
        os.mkdir(rawPath)
    outFileName=bename+"_"+os.path.basename(sys.argv[1])+".cbei";
    endReg = re.compile(r'cag|caa|cga', re.I)
    endRegM = re.compile(r'cca', re.I)
    pamReg = getPAMReg(beinfo[0])
    out=open(os.path.join(rawPath,outFileName),"w");
    for key in seqDict.keys():
        seq = str(seqDict[key].seq)
        seqr = tranStrand(seq)
        pos = pamReg.finditer(seq)
        posR = pamReg.finditer(seqr)
        plusRes = calSapcer(seq, pos, beinfo, endReg, 'Plus')
        minusRes = calSapcer(seqr, posR, beinfo, endRegM, 'Minus')
        plusRes.extend(minusRes)
        plusRes = sorted(plusRes, key=(lambda x: x['location']))
        for tres in plusRes:
            out.write(key+"\t"+"\t".join([
                str(tres['mode']),
                str(tres['location']),
                str(tres['detailSpacer']),
                str(tres['pureSpacer']),
                str(tres['spacerRange']),
                str(tres['editpos']),
                str(tres['editRange']),
                str(tres['pamSeq']),
                str(tres['pamRange']),
                str(tres['editpatten'])
            ])+"\n")
    print("CBEI calculation of base editor \""+bename+"\" , done!")

if __name__ == '__main__':
    beinfos = {
        "BE":["NGG",20,4,8,5],
        "YE1-BE3":["NGG",20,5,7,5],
        "EE-BE3":["NGG",20,5,6,5],
        "YEE-BE3":["NGG",20,6,6,5],
        "VQR-BE3":["NGAN",20,4,11,5],
        "VRER-BE3":["NGCG",20,3,10,5],
        "SaBE":["NNGRRT",21,3,12,5],
        "Sa(KKH)-BE3":["NNNRRT",21,3,12,5],
        "Cas12a–BE":["TTTV",20,10,12,3],
        "Target-AID":["NGG",20,2,4,5],
        "Target-AID-NG":["NG",20,2,4,5],
        "xBE3":["NG",20,4,8,5],
        "BE-PLUS":["NGG",20,4,14,5]
    }

    seqDict = SeqIO.index(sys.argv[1], "fasta")
    for key in beinfos:
        runBatch(key, beinfos[key], seqDict)

    print("Calculate complete!")




import os,re,random
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter
import numpy as np
from Bio.SeqUtils import GC
from cycler import cycler

def statCBEI(seqDict,cbeiPath,plotPath,statPath,fileName):
    transNum=len(seqDict)
    if (not os.path.exists(plotPath)):
        os.mkdir(plotPath)

    if (not os.path.exists(statPath)):
        os.mkdir(statPath)

    files=os.listdir(cbeiPath)
    allRaio={}
    for cfile in files:
        if(re.match(r'.*'+fileName+'.*\.cbei$',cfile,re.I)):
            tmp=cfile.split(".")
            be=tmp[0].split("_")[0]
            filePath=os.path.join(cbeiPath,cfile)
            cbeiInfo=getInfo(be,fileName,filePath,statPath)
            allRaio[be]=[len(cbeiInfo["t25"])/transNum,len(cbeiInfo["t5"])/transNum,len(cbeiInfo["t75"])/transNum,]        
            editablePie(cfile,transNum,cbeiInfo["t25"],cbeiInfo["t5"],cbeiInfo["t75"],plotPath)
    print("#"*20)
    print("Pie charts for different BEs have been generated. \nPath:")
    print("\t"+os.path.join(plotPath,"[BE names]_"+fileName+".statPie.png"))
    
    transGCStat(fileName,seqDict,plotPath)
    print("Transcript statistics and mapping completed.\nPath:\n\t"+os.path.join(plotPath,fileName+"_CDSlength.png"))
    print("\t"+os.path.join(plotPath,fileName+"_GCstats.png"))
    print("\t"+os.path.join(plotPath,fileName+"_CodonUsage.png"))
    print("\t"+os.path.join(plotPath,fileName+"_CDSlength.png"))

    statAllRatio(allRaio,plotPath,fileName)
    print("The comparison of CBEI ratio of different BE has been completed.\nPath:")
    print("\t"+os.path.join(plotPath,fileName+".statBar.png"))
    begROC(transNum, cbeiPath, plotPath,fileName)
    print("\t"+os.path.join(plotPath,fileName+".statROC.png"))
    
def begROC(transNum, cbeiPath, plotPath,fileName):
    beName={}
    files=os.listdir(cbeiPath)
    for file in files:
        if not (re.match(r".*"+fileName+".*\.cbei$",file,re.I)):
            continue
        title=file.strip().split(".")
        beName[title[0]]=[0 for k in range(0,100)]
        with open(os.path.join(cbeiPath,file)) as f:
            tgene={}
            for line in f:
                if (re.match(r'^#.*$',line,re.I)):
                    continue
                tsp=line.strip().split("\t")
                if (tsp[0] in tgene):
                    if (tsp[2]<tgene[tsp[0]]):
                        tgene[tsp[0]]=tsp[2]
                else:
                    tgene[tsp[0]]=tsp[2]
            for key in tgene.keys():
                tpos=int(float(tgene[key])*100)+1
                for p in range(tpos,100):
                    beName[title[0]][p]+=1
    statROC(beName,plotPath,fileName,transNum)

def to_percent(value, pos):
    return '%1.0f'%(100*value) + '%'

def statROC(beName,plotPath,fileName,transNum):
    nxarr= [[ 0 for j in range(0,100)]for i in beName.keys()]
    nyarr= [[ 0 for j in range(0,100)]for i in beName.keys()]
    thre=[ i/100 for i in range(0,100)]
    index=0
    names=[]
    for key in beName.keys():
        tname=key.split("_")
        names.append(tname[0])
        for i in range(0,100):
            beName[key][i]/=transNum
            nxarr[index][i]=thre[i]
            nyarr[index][i]=beName[key][i]
        index+=1
    plt.close()
    fig,ax=plt.subplots(figsize=(8,6))
    tlines=['-', '--', ':', '-.']
    for i in range(len(nxarr)):
        ax.plot(nxarr[i],nyarr[i],label=names[i],linestyle=tlines[random.randint(0,3)])
    ax.legend()
    ax.set_xlabel("Threshold")
    ax.set_ylabel("CBEI ratio")
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
    plt.savefig(os.path.join(plotPath,fileName+".statROC.png"),dpi=300)
    plt.close()
    

def statAllRatio(ratio,plotPath,fileName):
    thres=['0.25', '0.5', '0.75']
    xlab=[]
    thre25=[]
    thre5=[]
    thre75=[]
    x = np.arange(len(ratio))
    fig2, ax2 = plt.subplots(3,1, figsize=(8,12),sharex=True)

    for key in ratio:
        xlab.append(key)
        thre25.append(ratio[key][0])
        thre5.append(ratio[key][1])
        thre75.append(ratio[key][2])

    ax2[0].bar(xlab, thre25)
    ax2[0].set(title="Threshold=25%")
    ax2[0].set_xticklabels(xlab,rotation=45)
    autolabel(ax2[0],xlab,thre25)
    ax2[0].set_ylabel("CBEI ratio")
    ax2[1].bar(xlab, thre5)
    ax2[1].set_xticklabels(xlab,rotation=45)
    ax2[1].set_ylabel("CBEI ratio")
    autolabel(ax2[1],xlab,thre5)
    ax2[1].set(title="Threshold=50%")
    ax2[2].bar(xlab, thre75)
    autolabel(ax2[2],xlab,thre75)
    ax2[2].set_xticklabels(xlab,rotation=45)
    ax2[2].set_ylabel("CBEI ratio")
    ax2[2].set(title="Threshold=75%")
    ax2[0].get_yaxis().set_major_formatter(FuncFormatter(to_percent))
    ax2[1].get_yaxis().set_major_formatter(FuncFormatter(to_percent))
    ax2[2].get_yaxis().set_major_formatter(FuncFormatter(to_percent))
    plt.tight_layout()
    plt.savefig(os.path.join(plotPath,fileName+".statBar.png"),dpi=300)
    plt.close() 

def autolabel(plt,x,y):
    for xx, yy in zip(x,y):
        plt.text(xx, yy+0.01, "%1.2f" % (yy*100)+"%", ha='center')

def func(pct, allvals):
    absolute = int(pct/100.*np.sum(allvals))
    return "{:.2f}%\n{:d}".format(pct, absolute)


def editablePie(cfile, transNum,t25,t5,t75,plotPath):
    tmp=cfile.split(".")
    fileName="_".join(tmp[0].split("_")[1:])
    be=tmp[0].split("_")[0]
    labels = 'Editable', 'Others'
    explode = (0.1, 0)
    fig, axs = plt.subplots(1,3,figsize=(9, 4))
    # fig.subtitle=cfile
    data25=[len(t25),transNum-len(t25)]
    data5=[len(t5),transNum-len(t5)]
    data75=[len(t75),transNum-len(t75)]
    # dataBase25=statBase(t25)
    # dataBase5=statBase(t5)
    # dataBase75=statBase(t75)

    axs[0].pie(data25, explode=explode, labels=labels,autopct=lambda pct: func(pct, data25),
            shadow=False, startangle=90, textprops={'size': 'smaller'})
    axs[0].set(aspect="equal", title='Threshold=0.25')
    axs[1].pie(data5, explode=explode, labels=labels,autopct=lambda pct: func(pct, data5),
            shadow=False, startangle=90, textprops={'size': 'smaller'})
    axs[1].set(aspect="equal", title='Threshold=0.5')
    axs[2].pie(data75, explode=explode, labels=labels,autopct=lambda pct: func(pct, data75),
            shadow=False, startangle=90, textprops={'size': 'smaller'})
    axs[2].set(aspect="equal", title='Threshold=0.75')
    plt.suptitle("Editable transcripts ratio of "+be+" of "+fileName)
    plt.savefig(os.path.join(plotPath,be+"_"+fileName+".statPie.png"),dpi=300)
    plt.close()
    
def transGCStat(fileName,seqDict,plotPath):
    import re
    Codons = {
        "TTT": 0, "TTC": 0, "TTA": 0, "TTG": 0,
        "CTT": 0, "CTC": 0, "CTA": 0, "CTG": 0,
        "ATT": 0, "ATC": 0, "ATA": 0, "ATG": 0,
        "GTT": 0, "GTC": 0, "GTA": 0, "GTG": 0,
        "TAT": 0, "TAC": 0, "TAA": 0, "TAG": 0,
        "CAT": 0, "CAC": 0, "CAA": 0, "CAG": 0,
        "AAT": 0, "AAC": 0, "AAA": 0, "AAG": 0,
        "GAT": 0, "GAC": 0, "GAA": 0, "GAG": 0,
        "TCT": 0, "TCC": 0, "TCA": 0, "TCG": 0,
        "CCT": 0, "CCC": 0, "CCA": 0, "CCG": 0,
        "ACT": 0, "ACC": 0, "ACA": 0, "ACG": 0,
        "GCT": 0, "GCC": 0, "GCA": 0, "GCG": 0,
        "TGT": 0, "TGC": 0, "TGA": 0, "TGG": 0,
        "CGT": 0, "CGC": 0, "CGA": 0, "CGG": 0,
        "AGT": 0, "AGC": 0, "AGA": 0, "AGG": 0,
        "GGT": 0, "GGC": 0, "GGA": 0, "GGG": 0}
    gc_values=[]
    seqlength=[]
    cCount=0
    for key in seqDict.keys():
        seq = str(seqDict[key].seq)
        if len(seq) % 3 !=0 :
            continue
        for i in range(0, len(seq), 3):
            codon = seq[i : i + 3]
            if (codon in Codons):
                Codons[codon]+=1
                cCount+=1
        seqlength.append(len(seq))
        gc_values.append(GC(seq))
    gc_values=sorted(gc_values)
    plt.subplots(figsize=(6, 6))
    plt.plot(gc_values)
    plt.title("GC%% of %i CDSs \n from %.2f%% to %.2f%%" % (len(seqDict),min(gc_values),max(gc_values)))
    plt.ylabel("GC content (%)")
    plt.xlabel("CDSs")
    plt.savefig(os.path.join(plotPath,fileName+"_GCstats.png"),dpi=300)
    plt.close()
    seqlength=sorted(seqlength)
    plt.plot(seqlength)
    plt.title("CDSs length of %i CDSs \n from %int to %int" % (len(seqDict),min(seqlength),max(seqlength)))
    plt.ylabel("Sequene length (nt)")
    plt.xlabel("CDSs")
    plt.savefig(os.path.join(plotPath,fileName+"_CDSlength.png"),dpi=300)
    plt.close()
    codonName=[]
    codonFre=[]
    tcolor=[]
    for key in Codons.keys():
        Codons[key]=Codons[key]/cCount*100
        codonName.append(key)
        codonFre.append(Codons[key])
        if (re.match(r'CAG|CGA|CAA|CCA',key,re.I)):
            tcolor.append("tomato")
        else:
            tcolor.append("dodgerblue")
    fig3, ax3 = plt.subplots(figsize=(20, 5))
    ax3.bar(codonName, codonFre,color=tcolor)
    ax3.set(title="Codon frequency (%%) \n Frequency of CAG,CGA,CAA,CCA correspond to %.2f%%,%.2f%%,%.2f%%,%.2f%%" % (Codons["CAG"],Codons["CGA"],Codons["CAA"],Codons["CCA"]))
    ax3.set_xticklabels(codonName,rotation=45)
    plt.ylabel("Codon frequency (%)")
    plt.savefig(os.path.join(plotPath,fileName+"_CodonUsage.png"),dpi=300)
    plt.close()


def getInfo(be,fileName,filePath,statPath):
    cbeiTran25={}
    cbeiTran5={}
    cbeiTran75={}
    outf25=open(os.path.join(statPath,be+"_"+fileName+".Thre025.tsv"),"w")
    outf5=open(os.path.join(statPath,be+"_"+fileName+".Thre05.tsv"),"w")
    outf75=open(os.path.join(statPath,be+"_"+fileName+".Thre075.tsv"),"w")
    outf25.write("#Editable sites in the first 25% of the transcript\n")
    outf5.write("#Editable sites in the first 50% of the transcript\n")
    outf75.write("#Editable sites in the first 75% of the transcript\n")
    fheader="\t".join([
        "#Transcript",
        "Strand",
        "Position",
        "CBEIdetail",
        "Spacer",
        "SpacerRegion",
        "EditPosition",
        "EditWindowsRegion",
        "PAM",
        "PAMregion",
        "Pattern"
    ])
    outf25.write(fheader+"\n")
    outf5.write(fheader+"\n")
    outf75.write(fheader+"\n")
    with open(filePath,"r") as tcbei:
        for line in tcbei:
            tarr=line.strip().split("\t")
            tarr[2]=float(tarr[2])
            if tarr[2]<0.25:
                if tarr[0] in cbeiTran25:
                    continue
                else:
                    cbeiTran25[tarr[0]]=tarr[10]
                    outf25.write(line)
                if tarr[0] in cbeiTran5:
                    continue
                else:
                    cbeiTran5[tarr[0]]=tarr[10]
                    outf5.write(line)
                if tarr[0] in cbeiTran75:
                    continue
                else:
                    cbeiTran75[tarr[0]]=tarr[10]
                    outf75.write(line)
            elif tarr[2]<0.5:
                if tarr[0] in cbeiTran5:
                    continue
                else:
                    cbeiTran5[tarr[0]]=tarr[10]
                    outf5.write(line)
                if tarr[0] in cbeiTran75:
                    continue
                else:
                    cbeiTran75[tarr[0]]=tarr[10]
                    outf75.write(line)
            elif tarr[2]<0.75:
                if tarr[0] in cbeiTran75:
                    continue
                else:
                    cbeiTran75[tarr[0]]=tarr[10]
                    outf75.write(line)
    return {"t25":cbeiTran25,"t5":cbeiTran5,"t75":cbeiTran75}

if __name__ == "__main__":
    
    from Bio import SeqIO
    seqDict = SeqIO.index("../Bacillus_subtilis.ASM69118v1.cds.all.fa", "fasta")
    statCBEI(seqDict,"../CBEIRaw/","../CBEIPlot/","../CBEIRes")



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="autocbei",
    version="1.7",
    packages=find_packages(exclude=["cbeiUnitTests"]),
    description = '"autoCBEI.py" can calculate the potential CBEI loci of the target CDSs and perform statistics and plots.',
    author = "Haopeng Yu",
    author_email = "atlasbioin4@gmail.com",
    url = "https://github.com/atlasbioinfo/CRISPR-CBEI/tree/master/autoCBEI",
    python_requires='>=3.6.0',
    license='Apache-2.0',
    entry_points = {
        'console_scripts' : [
            'autocbei = autocbei:mainCBEI'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires = ["biopython","matplotlib"],
    package_data = {
        '':['*.fa']
    }
)



import os,re
from autocbei.cbei.autoCBEI import error

def judgeBEF(file):
    rules=["Please follow the rules, eg: .",
                    "BE,NGG,20,4,8,5",
                    "name, PAM, spacer length, edit beg, edit end, PAM at 5' or 3' of spacer"]
    if (os.path.isfile(file)):
        tbeinfo={}
        with open(file,"r") as f:
            for line in f:
                if (len(line.strip())<2):
                    continue
                tmp=line.strip().split(",")
                if(len(tmp)!=6):
                    error(["An incomplete line has been detected in your BE input file (list bellow), please complete it:",line.strip()])
                if (tmp[0] not in tbeinfo):
                    tbeinfo[tmp[0]]=tmp[1:]
                else:
                    error([
                        "A duplicate base editors appears in the file, please check:",
                        tmp[0]+"\t"+",".join(tbeinfo[tmp[0]]),
                        ",".join(tmp)
                    ])
                    
        for tbe in tbeinfo:
            if (not re.match(r'^[ATCGNRYMKSWHBVD]+$',tbeinfo[tbe][0],re.I)):
                error([
                    "The PAM sequence of this record",
                    "\t"+tbe+","+",".join(tbeinfo[tbe]),
                    "in the Base editor file does not match the requirement.",
                    "Please indicate with ATCGNRYMKSWHBVD."
                ])
            if (not (tbeinfo[tbe][1].isdecimal() and 
                    tbeinfo[tbe][2].isdecimal() and
                    tbeinfo[tbe][3].isdecimal() and 
                    tbeinfo[tbe][4].isdecimal())):
                error([
                    "The parameters of base editor:",
                    "\t"+tbe+","+",".join(tbeinfo[tbe]),
                    "has non-integers in it. ",
                    "\n".join(rules)
                ])
            tbeinfo[tbe][1]=int(tbeinfo[tbe][1])
            tbeinfo[tbe][2]=int(tbeinfo[tbe][2])
            tbeinfo[tbe][3]=int(tbeinfo[tbe][3])
            tbeinfo[tbe][4]=int(tbeinfo[tbe][4])
            if (tbeinfo[tbe][2]>tbeinfo[tbe][3]):
                error([
                    "There's an error in the parameters of the basic editor:",
                    "\t"+tbe+","+",".join(tbeinfo[tbe]),
                    "The begin pos: "+tbeinfo[tbe][2]+" is larger than the end"+tbeinfo[tbe][3],
                    "\n".join(rules)
                ])
            if (tbeinfo[tbe][2]>=tbeinfo[tbe][1] or tbeinfo[tbe][3]>=tbeinfo[tbe][1]):
                error([
                    "There's an error in the parameters of the basic editor:",
                    "\t"+tbe+","+",".join(tbeinfo[tbe]),
                    "The begin pos: "+tbeinfo[tbe][2]+" or the end"+tbeinfo[tbe][3]+" is larger than the spacer length "+ str(tbeinfo[tbe][1]),
                    "\n".join(rules)
                ])
            if (not (tbeinfo[tbe][4]==5 or tbeinfo[tbe][4]==3)):
                error([
                    "There's an error in the parameters of the basic editor:",
                    "\t"+tbe+","+",".join(tbeinfo[tbe]),
                    "The direction should be 5 or 3, not "+tbeinfo[tbe][4],
                    "\n".join(rules)
                ])
        return tbeinfo               
    error([
        "The "+file+" is not exists, please check!"
    ]) 
        